/*
using System.IO;
using UnityEditor;
using UnityEngine;
using UnityEngine.XR;
using Passer.Tracking;

namespace Passer {
    using Humanoid;

    public class Oculus_Editor : Tracker_Editor {

#if hOCULUS && (UNITY_STANDALONE_WIN || UNITY_ANDROID)

#if UNITY_ANDROID && hOCHAND
        private readonly static System.Version minHandTrackingVersion = new System.Version(1, 44, 0);
#endif

        #region Tracker

        public class TrackerProps : HumanoidControl_Editor.HumanoidTrackerProps {

            private readonly SerializedProperty trackerProp;
#if UNITY_ANDROID
            SerializedProperty deviceTypeProp;
            SerializedProperty handTrackingProp;
#endif

            public TrackerProps(SerializedObject serializedObject, HumanoidControl_Editor.HumanoidTargetObjs targetObjs, OculusTracker _oculus)
                : base(serializedObject, targetObjs, _oculus, "oculus") {
                tracker = _oculus;

                trackerProp = serializedObject.FindProperty("oculus.tracker");

                if (targetObjs.headTargetObj != null)
                    headSensorProp = targetObjs.headTargetObj.FindProperty(nameof(HumanoidControl.oculus));
                if (targetObjs.leftHandTargetObj != null)
                    leftHandSensorProp = targetObjs.leftHandTargetObj.FindProperty(nameof(HumanoidControl.oculus));
                if (targetObjs.rightHandTargetObj != null)
                    rightHandSensorProp = targetObjs.rightHandTargetObj.FindProperty(nameof(HumanoidControl.oculus));

#if UNITY_ANDROID && !UNITY_2020_1_OR_NEWER
                handTrackingProp = serializedObject.FindProperty("oculus.handTracking");
                //deviceTypeProp = serializedObject.FindProperty("oculus.androidDeviceType");

                //OculusHumanoidTracker.AndroidDeviceType androidDeviceType = (OculusHumanoidTracker.AndroidDeviceType)deviceTypeProp.intValue;
                //if (androidDeviceType == OculusHumanoidTracker.AndroidDeviceType.OculusQuest)
                //    CheckQuestManifest();
#endif
            }

            public override void Inspector(HumanoidControl humanoid) {
                //bool oculusSupported = OculusSupported();
                //if (oculusSupported) {
                //    humanoid.oculus.CheckTracker(humanoid);
                //    humanoid.headTarget.oculus.CheckSensor(humanoid.headTarget);
                //    humanoid.leftHandTarget.oculus.CheckSensor(humanoid.leftHandTarget);
                //    humanoid.rightHandTarget.oculus.CheckSensor(humanoid.rightHandTarget);
#if !pUNITYXR
                Inspector(humanoid, "Oculus");

                if (enabledProp.boolValue) {
                    EditorGUI.indentLevel++;
                    if (trackerProp.objectReferenceValue == null) {
                        // Tracker does not exit
                        using (new EditorGUILayout.HorizontalScope()) {
                            EditorGUILayout.LabelField("Tracker", GUILayout.Width(120));
                            if (GUILayout.Button("Show")) {
                                humanoid.oculus.CheckTracker(humanoid);
                            }
                        }
                    }
                    else
                        trackerProp.objectReferenceValue = (Oculus)EditorGUILayout.ObjectField("Tracker", trackerProp.objectReferenceValue, typeof(Oculus), true);
                    EditorGUI.indentLevel--;
                }
#if UNITY_ANDROID && hOCHAND && !UNITY_2020_1_OR_NEWER
                    EditorGUI.indentLevel++;
                    deviceTypeProp.intValue = (int)(OculusHumanoidTracker.AndroidDeviceType)EditorGUILayout.EnumPopup("Device Type", (OculusHumanoidTracker.AndroidDeviceType)deviceTypeProp.intValue);
                    handTrackingProp.boolValue = EditorGUILayout.Toggle("Hand Tracking", handTrackingProp.boolValue);
#if UNITY_EDITOR_OSX
                    if (handTrackingProp.boolValue) {
                        EditorGUILayout.HelpBox(
                            "Hand tracking required at least OVR Plugin version 1.44 or higher\n" +
                            "Install the latetest version using the Oculus Integration package",
                            MessageType.Warning);
                    }
#else
                    if (handTrackingProp.boolValue && !(Humanoid.Tracking.OculusDevice.version >= minHandTrackingVersion)) {
                        EditorGUILayout.HelpBox(
                            "Hand tracking required at least OVR Plugin version 1.44 or higher\n" +
                            "Install the latetest version using the Oculus Integration package",
                            MessageType.Error);
                    }
#endif
                    EditorGUI.indentLevel--;
#endif
                //}
                //else
                //    enabledProp.boolValue = false;
#endif
            }

            protected virtual void CheckQuestManifest() {
                string manifestPath = Application.dataPath + "/Plugins/Android/AndroidManifest.xml";
                FileInfo fileInfo = new FileInfo(manifestPath);
                fileInfo.Directory.Create();
                bool manifestAvailable = File.Exists(manifestPath);
                if (manifestAvailable)
                    return;

                string humanoidPath = Configuration_Editor.FindHumanoidFolder();
                string questManifestPath = Application.dataPath + humanoidPath + "Extensions/Oculus/QuestManifest.xml";
                File.Copy(questManifestPath, manifestPath);
            }
        }

        #endregion

        #region Head

        public class HeadTargetProps : HeadTarget_Editor.TargetProps {


            private readonly SerializedProperty hmdProp;
            SerializedProperty overrideOptitrackPositionProp;

            public HeadTargetProps(SerializedObject serializedObject, HeadTarget headTarget)
                : base(serializedObject, headTarget.oculus, headTarget, "oculus") {

                hmdProp = serializedObject.FindProperty("oculus.hmd");
                overrideOptitrackPositionProp = serializedObject.FindProperty("oculus.overrideOptitrackPosition");
            }

            public override void Inspector() {
                if (!headTarget.humanoid.oculus.enabled)
                    return;

                enabledProp.boolValue = HumanoidTarget_Editor.ControllerInspector(headTarget.oculus, headTarget);
                headTarget.oculus.enabled = enabledProp.boolValue;

                if (enabledProp.boolValue) {
                    EditorGUI.indentLevel++;
                    if (headTarget.oculus.hmd == null) {
                        // Hmd does not exist
                        using (new EditorGUILayout.HorizontalScope()) {
                            EditorGUILayout.LabelField("Hmd", GUILayout.Width(120));
                            if (GUILayout.Button("Show")) {
                                headTarget.oculus.CheckSensor(headTarget);
                            }
                        }
                    }
                    else
                        hmdProp.objectReferenceValue = (OculusHmd)EditorGUILayout.ObjectField("Hmd", hmdProp.objectReferenceValue, typeof(OculusHmd), true);
#if hOPTITRACK
                    if (headTarget.optitrack.enabled)
                        overrideOptitrackPositionProp.boolValue = EditorGUILayout.Toggle("Override OptiTrack Position", overrideOptitrackPositionProp.boolValue);
                    else
#endif
                    overrideOptitrackPositionProp.boolValue = true;

                    EditorGUI.indentLevel--;
                }
                else
                    headTarget.oculus.CheckSensor(headTarget);

            }
        }

        #region HMD Component
        [CustomEditor(typeof(OculusHmd))]
        public class OculusHmdComponent_Editor : Editor {
            OculusHmd sensorComponent;

            private void OnEnable() {
                sensorComponent = (OculusHmd)target;
            }

            public override void OnInspectorGUI() {
                serializedObject.Update();

                EditorGUI.BeginDisabledGroup(true);
                EditorGUILayout.EnumPopup("Status", sensorComponent.status);
                EditorGUILayout.FloatField("Position Confidence", sensorComponent.positionConfidence);
                EditorGUILayout.FloatField("Rotation Confidence", sensorComponent.rotationConfidence);
                EditorGUI.EndDisabledGroup();

                serializedObject.ApplyModifiedProperties();
            }
        }
        #endregion

        #endregion

        #region Hand
        public class HandTargetProps : HandTarget_Editor.TargetProps {

#if UNITY_ANDROID && hOCHAND
            SerializedProperty handTrackingProp;
            SerializedProperty skeletonProp;
#endif
#if pUNITYXR
            SerializedProperty controllerProp;
#endif

            public HandTargetProps(SerializedObject serializedObject, HandTarget handTarget)
                : base(serializedObject, handTarget.oculus, handTarget, "oculus") {

#if UNITY_ANDROID && hOCHAND
                handTrackingProp = serializedObject.FindProperty("oculus.handTracking");
                skeletonProp = serializedObject.FindProperty("oculus.handSkeleton");
#endif
#if pUNITYXR
                controllerProp = serializedObject.FindProperty("oculus.unityXRController");
#endif
            }

            public override void Inspector() {
                if (!handTarget.humanoid.oculus.enabled || !OculusSupported())
                    return;

#if !pUNITYXR
                CheckControllerComponent(handTarget);
#if UNITY_ANDROID && hOCHAND
                handTarget.oculus.CheckSensor(handTarget, handTarget.humanoid.oculus);
#endif

                enabledProp.boolValue = HumanoidTarget_Editor.ControllerInspector(handTarget.oculus, handTarget);
                handTarget.oculus.enabled = enabledProp.boolValue;
#if !pUNITYXR
                handTarget.oculus.CheckSensorTransform();
#endif
                if (!Application.isPlaying) {
                    handTarget.oculus.SetSensor2Target();
                    handTarget.oculus.ShowSensor(handTarget.humanoid.showRealObjects && handTarget.showRealObjects);
                }

                if (enabledProp.boolValue) {
                    EditorGUI.indentLevel++;
#if pUNITYXR
                    controllerProp.objectReferenceValue = (UnityXRController)EditorGUILayout.ObjectField("Controller", controllerProp.objectReferenceValue, typeof(UnityXRController), true);
#else
                    sensorTransformProp.objectReferenceValue = (Transform)EditorGUILayout.ObjectField("Tracker Transform", handTarget.oculus.sensorTransform, typeof(Transform), true);
#endif
#if UNITY_ANDROID && hOCHAND
                    if (handTarget.humanoid.oculus.handTracking) {
                        skeletonProp.objectReferenceValue = (OculusHandSkeleton)EditorGUILayout.ObjectField("Skeleton", skeletonProp.objectReferenceValue, typeof(OculusHandSkeleton), true);
                    }
#endif
                    EditorGUI.indentLevel--;
                }
#endif
            }

            protected static void CheckControllerComponent(HandTarget handTarget) {
                if (handTarget.oculus.sensorTransform == null)
                    return;

#if pUNITYXR
                //                UnityXRController controller = handTarget.oculus.sensorTransform.GetComponent<UnityXRController>();
                //                if (controller == null)
                //                    controller = handTarget.oculus.sensorTransform.gameObject.AddComponent<UnityXRController>();
                //                controller.isLeft = handTarget.isLeft;
                //#else
                OculusController sensorComponent = handTarget.oculus.sensorTransform.GetComponent<OculusController>();
                if (sensorComponent == null)
                    sensorComponent = handTarget.oculus.sensorTransform.gameObject.AddComponent<OculusController>();
                sensorComponent.isLeft = handTarget.isLeft;
#endif
            }

            //#if hOCHAND
            //            protected static void CheckSkeletonComponent(HandTarget handTarget) {
            //                if (handTarget.oculus.handSkeleton == null) {
            //                    handTarget.oculus.handSkeleton = handTarget.oculus.FindHandSkeleton(handTarget.isLeft);
            //                    if (handTarget.oculus.handSkeleton == null)
            //                        handTarget.oculus.handSkeleton = handTarget.oculus.CreateHandSkeleton(handTarget.isLeft, handTarget.showRealObjects);
            //                }

            //            }
            //#endif
        }
        #region Controller Component
        [CustomEditor(typeof(OculusController))]
        public class OculusControllerComponent_Editor : Editor {
            OculusController controllerComponent;

            private void OnEnable() {
                controllerComponent = (OculusController)target;
            }

            public override void OnInspectorGUI() {
                serializedObject.Update();

                EditorGUI.BeginDisabledGroup(true);
                EditorGUILayout.EnumPopup("Status", controllerComponent.status);
                EditorGUILayout.FloatField("Position Confidence", controllerComponent.positionConfidence);
                EditorGUILayout.FloatField("Rotation Confidence", controllerComponent.rotationConfidence);
                EditorGUILayout.Space();
                EditorGUILayout.Toggle("Is Left", controllerComponent.isLeft);
                EditorGUILayout.Vector3Field("Joystick", controllerComponent.primaryAxis);
                EditorGUILayout.Slider("Index Trigger", controllerComponent.trigger1, -1, 1);
                EditorGUILayout.Slider("Hand Trigger", controllerComponent.trigger2, -1, 1);
                if (controllerComponent.isLeft) {
                    EditorGUILayout.Slider("Button X", controllerComponent.button1, -1, 1);
                    EditorGUILayout.Slider("Button Y", controllerComponent.button2, -1, 1);
                }
                else {
                    EditorGUILayout.Slider("Button A", controllerComponent.button1, -1, 1);
                    EditorGUILayout.Slider("Button B", controllerComponent.button2, -1, 1);
                }
                EditorGUILayout.Slider("Thumbrest", controllerComponent.button3, -1, 1);
                EditorGUI.EndDisabledGroup();

                serializedObject.ApplyModifiedProperties();
            }
        }
        #endregion
        #endregion

        #region Object Target
        
                private static SerializedProperty objectEnabledProp;
                private static SerializedProperty objectSensorTransformProp;
                private static SerializedProperty objectSensor2TargetPositionProp;
                private static SerializedProperty objectSensor2TargetRotationProp;

                public static void InitObject(SerializedObject serializedObject, ObjectTarget objectTarget) {
                    objectEnabledProp = serializedObject.FindProperty("oculusController.enabled");
                    objectSensorTransformProp = serializedObject.FindProperty("oculusController.sensorTransform");
                    objectSensor2TargetPositionProp = serializedObject.FindProperty("oculusController.sensor2TargetPosition");
                    objectSensor2TargetRotationProp = serializedObject.FindProperty("oculusController.sensor2TargetRotation");

                    objectTarget.oculus.Init(objectTarget);
                }

                private enum LeftRight {
                    Left,
                    Right
                }

                public static void ObjectInspector(OculusController controller) {
                    objectEnabledProp.boolValue = Target_Editor.ControllerInspector(controller);
                    controller.CheckSensorTransform();

                    if (objectEnabledProp.boolValue) {
                        EditorGUI.indentLevel++;
                        LeftRight leftRight = controller.isLeft ? LeftRight.Left : LeftRight.Right;
                        leftRight = (LeftRight)EditorGUILayout.EnumPopup("Tracker Id", leftRight);
                        controller.isLeft = leftRight == LeftRight.Left;
                        objectSensorTransformProp.objectReferenceValue = (Transform)EditorGUILayout.ObjectField("Tracker Transform", controller.sensorTransform, typeof(Transform), true);
                        EditorGUI.indentLevel--;
                    }
                }

                public static void SetSensor2Target(OculusController controller) {
                    controller.SetSensor2Target();
                    objectSensor2TargetRotationProp.quaternionValue = controller.sensor2TargetRotation;
                    objectSensor2TargetPositionProp.vector3Value = controller.sensor2TargetPosition;
                }
                
        #endregion
#endif
        public static bool OculusSupported() {
#if pUNITYXR
            return true;
#else
            string[] supportedDevices = XRSettings.supportedDevices;
            foreach (string supportedDevice in supportedDevices) {
                if (supportedDevice == "Oculus")
                    return true;
            }
            return false;
#endif
        }

    }

#if !hOCULUS
    [InitializeOnLoad]
    public class OculusCleanup {

        // Cleanup Oculus devices when Oculus is disabled in the preferences
        static OculusCleanup() {
            OculusHmd[] hmds = Object.FindObjectsOfType<OculusHmd>();
            foreach (OculusHmd hmd in hmds)
                Object.DestroyImmediate(hmd.gameObject, true);

            OculusController[] controllers = Object.FindObjectsOfType<OculusController>();
            foreach (OculusController controller in controllers)
                Object.DestroyImmediate(controller.gameObject, true);
        }
    }
#endif
}
*/